
;*********************************************************************************
;* editor.s:
;*********************************************************************************

;********************************************************************************
;* This file is part of gibl:							*
;*	Gordons Interactive Basic Language                                      *
;********************************************************************************
;*    A Tiny Basic for the 6502 inspired by the NIBL Tiny Basic interpreter;	*
;*    "National Industrial Basic Language" originally for the INS8060 SC/MP	*
;*    system.									*
;*										*
;*    gibl is distributed under a "Source Available" license.			*
;*	It is NOT Open source and must not be treated as such.			*
;*										*
;*    See the file LICENSE for details.						*
;*										*
;*    gibl is Copyright (c) 2023 by Gordon Henderson				*
;********************************************************************************

	.macpack	generic

	.include	"data.h"

	.include	"editor.h"


;********************************************************************************
;* editor:
;*	Insert/Replace/Delete a line in the program text.
;********************************************************************************
;* Pointer to line buffer is in svCursor.
;* Cursor points to the insertion point in the text.
;*
;* Each line is stored in the following format:
;*    Two bytes containing the line number (in binary, high byte first)
;*    One byte containing the length.
;*    The line text itself terminated by a CR.
;*    The last line is followed by $FF
;********************************************************************************

.proc	editor

; Copy line number out of hi/lo into hiLine/loLine

	lda	lo
	sta	loLine
	lda	hi
	sta	hiLine

; Get line length

	ldx	#4			; Start at 4 - line number, len, CR
	ldy	#0
:	lda	(svCursor),y
	cmp	#$0D
	beq	:+
	inx
	iny
	bne	:-
:

; If length is 4 and not an exact match from FLBL above, then we can return
;	(Trying to delete a non-existant line)

	cpx	#4
	bne	notEmptyLine

; OK. Length is 4. Was it an exact match?

	lda	lablHi
	bpl	exact4

; Not an exact match, so trying to delete a non-existant line, so...

	rts		

; It's a zero length line that exists, so ...

exact4:
	jsr	deleteLine
	rts

; Line length not 4, so...

notEmptyLine:
	lda	lablHi			; If line does not exists then we insert new, else ...
	bmi	insertNew
	jsr	deleteLine		; ... and fall into insert
.endproc


;********************************************************************************
;* insertNew:
;*	Insert a new line
;********************************************************************************

.proc	insertNew

; Make space for the new line

; cursor points to the first byte of RAM to use,
;	'top' is stored in topL/topH

; Copy top to P2

	lda	topL
	sta	p2L
	lda	topH
	sta	p2H

; P3 = Line length + P2

	clc
	txa
	adc	p2L
	sta	p3L
	lda	p2H
	adc	#0
	sta	p3H

; New Top is P3

	lda	p3L
	sta	topL
	lda	p3H
	sta	topH

; Move upwards in reverse from P2 to P3 until P2 = the cursor position.

	ldy	#0
nextByte:
	lda	(p2),y
	sta	(p3),y

	lda	p2L
	cmp	cursorL
	beq	testHi
decs:
	lda	p2L
	bne	:+
	dec	p2H
:	dec	p2L

	lda	p3L
	bne	:+
	dec	p3H
:	dec	p3L

	jmp	nextByte

testHi:
	lda	p2H
	cmp	cursorH
	bne	decs

; Now, copy the line into @cursor ...
; Copy line number and length
;	Remember - line number is stored HIGH byte first.

	ldy	#0
	lda	hiLine
	sta	(cursor),y
	iny
	lda	loLine
	sta	(cursor),y
	iny
	txa			; Length
	sta	(cursor),y

;; Optimise - copy until CR?
	dex			; Remove the +4 we started with
	dex
	dex
	dex

; Adjust cursor so we have a zero index

	clc
	lda	cursorL
	adc	#3
	sta	cursorL
	bcc	:+
	inc	cursorH
:

	ldy	#0
:	lda	(svCursor),y	; Input
	sta	(cursor),y	; Store
	iny
	dex
	bpl	:-

	rts
.endproc


;********************************************************************************
; deleteLine:
;	Delete the line at the cursor...
;********************************************************************************

.proc	deleteLine

; Move from cursor+len to cursor until we hit the FF
;	subract len from top

	ldy	#2
	lda	(cursor),y		; Get len
	tay

; Copy cursor to P2 and cursor+len into P3

	lda	cursorL
	sta	p2L
	lda	cursorH
	sta	p2H

	tya
	clc
	adc	cursorL
	sta	p3L
	lda	cursorH
	adc	#0
	sta	p3H

	ldy	#0
copyLoop:
	lda	(p3),y
	sta	(p2),y
	cmp	#$FF
	beq	copyDone

	inc	p3L
	bne	:+
	inc	p3H
:
	inc	p2L
	bne	:+
	inc	p2H
:
	bne	copyLoop

; p2 is the new top

copyDone:
	lda	p2L
	sta	topL
	lda	p2H
	sta	topH

; and we're done

	rts
.endproc
